package com.winhc.phoenix.example.controller;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.winhc.phoenix.example.configuration.DynamicElasticSearchClient;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.SpanNearQueryBuilder;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * @author: XuJiakai
 * 2021/9/27 17:46
 */
@Api(tags = "法律咨询问题分类", value = "legal-consulting-clazz")
@AllArgsConstructor
@RestController
@RequestMapping("legal-consulting-clazz")
public class LegalConsultingClazzController {
    private final DynamicElasticSearchClient dynamicElasticSearchClient;

    private static final String index = "winhc-legal-consulting-clazz_v2";
    private static final String type = "_doc";
    private ObjectMapper mapper;
    private static final TypeReference<HashMap<String, Object>> typeRef
            = new TypeReference<HashMap<String, Object>>() {
    };

   /* public List<String> analyze(@RequestBody String content){
        RestClient restClient = dynamicElasticSearchClient.getRestClientByIndex(index);
        restClient.performRequest("GET",index+"/_analyze",)

    }*/


    @SneakyThrows
    @PostMapping("clazz")
    @ApiOperation(value = "问题分类")
    public Object query(@RequestBody String content) {
        RestHighLevelClient restHighLevelClient = dynamicElasticSearchClient.getRestHighLevelClientByIndex(index);


        BoolQueryBuilder boolQueryBuilder = boolQuery();

        boolQueryBuilder.should(multiMatchQuery(content).boost(2)
                .field("clazz_1", (float) 0.8)
                .field("clazz_2", (float) 0.5)
        );
        boolQueryBuilder.should(
                disMaxQuery()
                        .add(multiMatchQuery(content).minimumShouldMatch("4")
                                .boost((float) 0.5)
                                .field("clazz_1.standard", (float) 0.8)
                                .field("clazz_2.standard", (float) 0.2)
                        )
                        .add(multiMatchQuery(content).minimumShouldMatch("2")
                                .boost((float) 0.5)
                                .field("clazz_1.standard_dup", (float) 0.8)
                                .field("clazz_2.standard_dup", (float) 0.2)
                        )
                        .tieBreaker((float) 0.1)
        );

        HighlightBuilder highlightBuilder = new HighlightBuilder()
                .preTags("<font class='red'>")
                .postTags("</font>")
                .field("*")
               ;
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder()
                .query(boolQueryBuilder)
                .highlighter(highlightBuilder);
        SearchRequest searchRequest = new SearchRequest()
                .indices(index)
                .types(type)
                .source(searchSourceBuilder)
                ;

        SearchResponse search = restHighLevelClient.search(searchRequest);

        return mapper.readValue(search.toString(), typeRef);
       /* SearchHits hits = search.getHits();
        SearchHit[] hits1 = hits.getHits();
        ArrayList<String> strings = new ArrayList<>();

        for (SearchHit hit : hits1) {
            Map<String, Object> sourceAsMap = hit.getSourceAsMap();
            String name = (String) sourceAsMap.get("name");
            strings.add(name);
        }
        if (strings.isEmpty()) {
            strings.add("综合");
        }
        return strings;*/
    }

    private static QueryBuilder getSpanNearQuery(String fields, String content) {
        SpanNearQueryBuilder spanNearQueryBuilder = spanNearQuery(spanTermQuery(fields, content.charAt(0) + ""), 1);
        for (int i = 1; i < content.length(); i++) {
            spanNearQueryBuilder.addClause(spanTermQuery(fields, content.charAt(i) + ""));
        }
        return spanNearQueryBuilder;
    }


}
