package com.winhc.phoenix.example.controller;

import com.winhc.phoenix.example.aspect.Timer;
import com.winhc.phoenix.example.enums.CompanyQueryType;
import com.winhc.phoenix.example.enums.CompanySearchSortType;
import com.winhc.phoenix.example.enums.EsVersion;
import com.winhc.phoenix.example.service.CompanyDynamicService;
import com.winhc.phoenix.example.service.SearchService;
import com.winhc.phoenix.example.service.impl.FindRelationshipService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * @author: XuJiakai
 * 2020/11/19 14:51
 */
@Slf4j
@AllArgsConstructor
@RestController
@Api(tags = "es查询", value = "es")
@RequestMapping("es")
public class SearchController {
    private final Map<String, SearchService> map;
    private final CompanyDynamicService companyDynamicService;
    private final FindRelationshipService findRelationshipService;

    @Timer
    @ApiOperation(value = "es搜索")
    @GetMapping("query")
    public Object query(String content
            , @RequestParam(defaultValue = "v9") EsVersion version
            , @RequestParam(defaultValue = "0") int from
            , @RequestParam(defaultValue = "10") int size
            , @RequestParam(defaultValue = "默认排序") CompanySearchSortType sortType
    ) {
        if (content.startsWith("tips:")) {
            content = content.substring(5);
            log.info("search tips: {}", content);
            return map.get(version.getValue()).tips(content);
        }

        return map.get(version.getValue()).query(content, from, size, new HashSet<>(), sortType);
    }

    @Timer
    @ApiOperation(value = "搜索提示")
    @GetMapping("tips")
    public Object tips(String content
            , @RequestParam(defaultValue = "v9") EsVersion version
    ) {
        return map.get(version.getValue()).tips(content);
    }


    @Timer
    @ApiOperation(value = "找关系搜索")
    @GetMapping("find-relationship")
    public Object findRelationship(String content, @RequestParam(required = false) Set<CompanyQueryType> set, @RequestParam(defaultValue = "0") int from, @RequestParam(defaultValue = "10") int size) {
        if (set == null) {
            set = Collections.emptySet();
        }
        return map.get(EsVersion.v8版_Fast.getValue()).query(content, from, size, set, CompanySearchSortType.默认排序);
//        return findRelationshipService.findRelationship(content, typeList, from, size);
    }


    @Timer
    @ApiOperation(value = "企业动态")
    @GetMapping("dynamic")
    public Object dynamic(@RequestParam String companyId, @RequestParam(defaultValue = "0") int from, @RequestParam(defaultValue = "10") int size) {
        return companyDynamicService.list(companyId, null, from, size);
    }

    @Timer
    @ApiOperation(value = "企业动态-展开折叠")
    @GetMapping("dynamic-detail")
    public Object dynamicDetail(@RequestParam String companyId, @RequestParam String collapseKey, @RequestParam(defaultValue = "0") int from, @RequestParam(defaultValue = "10") int size) {
        return companyDynamicService.detail(companyId, collapseKey, from, size);
    }


}
