package com.winhc.phoenix.example.dao.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.winhc.phoenix.example.configuration.DynamicElasticSearchClient;
import com.winhc.phoenix.example.dao.SearchDao;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.util.EntityUtils;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.delete.DeleteRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.rescore.QueryRescorerBuilder;
import org.elasticsearch.search.rescore.RescoreBuilder;
import org.elasticsearch.search.sort.SortBuilder;
import org.springframework.stereotype.Repository;

import java.util.*;

/**
 * @author: XuJiakai
 * 2020/11/20 15:16
 */
@Slf4j
@Repository
@AllArgsConstructor
public class SearchDaoImpl implements SearchDao {
    private final DynamicElasticSearchClient dynamicElasticSearchClient;


    private final static Set<String> oldEsIndex = new HashSet<String>(

    ) {
        {
            add("winhc-company-dynamic-v1");
            add("wenshu_detail4");
        }
    };

    private RestClient getRestClient(String index) {
        if (oldEsIndex.contains(index)) {
            return dynamicElasticSearchClient.getRestClient("old");
        } else {
            return dynamicElasticSearchClient.getRestClient("new");
        }
    }

    private RestHighLevelClient getRestHighLevelClient(String index) {
        if (oldEsIndex.contains(index)) {
            return dynamicElasticSearchClient.getRestHighLevelClient("old");
        } else {
            return dynamicElasticSearchClient.getRestHighLevelClient("new");
        }
    }


    private ObjectMapper mapper;
    private static final TypeReference<HashMap<String, Object>> typeRef
            = new TypeReference<HashMap<String, Object>>() {
    };

    @SneakyThrows
    @Override
    public Object search(String index, String type, QueryBuilder query, SortBuilder sortBuilder, FetchSourceContext fetchSourceContext, int from, int size) {
        return search(index, type, query, null, sortBuilder, fetchSourceContext, from, size, null);
    }

    @Override
    public Object search(String index, String type, QueryBuilder query, int from, int size) {
        return search(index, type, query, null, null, from, size);
    }

    @Override
    public Object search(String index, String type, QueryBuilder query, List<QueryRescorerBuilder> rescoreBuilder, SortBuilder sortBuilder, FetchSourceContext fetchSourceContext, int from, int size, String preference) {
        List<String> list = Arrays.asList(
                "cname.show",
                "cname.show.keyword",
                "name_alias",
                "cname.show.pinyin",
                "history_name.show.pinyin",
                "history_name.show",
                "legal_entity_name",
                "holder_history.name",
                "holder.name",
                "staff.name",
                "staff_history.name",
                "icp",
                "app_info",
                "company_tm",
                "emails",
                "emails.keyword",
                "phones",
                "icp_domain",
                "icp_domain.keyword",
                "reg_number",
                "reg_location",
                "org_number",
                "credit_code"
        );
        ArrayList<SortBuilder> objects = new ArrayList<>();


        if (sortBuilder != null) {
            objects.add(sortBuilder);
        }
        return search(index, type, query, rescoreBuilder, objects, fetchSourceContext, from, size, preference, list);
    }

    @SneakyThrows
    @Override
    public Object search(String index
            , String type
            , QueryBuilder query
            , List<QueryRescorerBuilder> rescoreBuilder
            , List<SortBuilder> sortBuilder
            , FetchSourceContext fetchSourceContext
            , int from
            , int size
            , String preference
            , List<String> highlightField
    ) {

        HighlightBuilder highlightBuilder = new HighlightBuilder().order(HighlightBuilder.Order.SCORE)
                .preTags("<font class='my-bold'>")
                .postTags("</font>")
                .order("score");
        if (highlightField != null)
            for (String f : highlightField) {
                highlightBuilder.field(f);
            }
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder()
                .query(query)
                .from(from)
                .size(size)
                .highlighter(highlightBuilder);

        if (rescoreBuilder != null && !rescoreBuilder.isEmpty()) {
            for (RescoreBuilder builder : rescoreBuilder) {
                searchSourceBuilder.addRescorer(builder);
            }
        }
        if (fetchSourceContext != null) {
            searchSourceBuilder.fetchSource(fetchSourceContext);
        }
        if (sortBuilder != null) {
            for (SortBuilder builder : sortBuilder) {
                searchSourceBuilder.sort(builder);
            }
        }

        SearchRequest searchRequest = new SearchRequest()
                .indices(index)
                .types(type)
                .source(searchSourceBuilder);
        if (StringUtils.isNotBlank(preference)) {
            searchRequest.preference(preference);
        }
        RestHighLevelClient restHighLevelClient = getRestHighLevelClient(index);

        SearchResponse search = restHighLevelClient.search(searchRequest);
        return mapper.readValue(search.toString(), typeRef);
    }

    @SneakyThrows
    @Override
    public Object search(String index, String type, SearchSourceBuilder searchSourceBuilder) {
        SearchRequest searchRequest = new SearchRequest()
                .indices(index)
                .types(type)
                .source(searchSourceBuilder);
        RestHighLevelClient restHighLevelClient = dynamicElasticSearchClient.getRestHighLevelClientByIndex(index);
        SearchResponse search = restHighLevelClient.search(searchRequest);
        return mapper.readValue(search.toString(), typeRef);
    }

    @SneakyThrows
    @Override
    public boolean deleteByIds(String index, String type, List<String> ids) {
        BulkRequest bulkRequest = new BulkRequest();
        for (String id : ids) {
            DeleteRequest deleteRequest = new DeleteRequest(index, type, id);
            bulkRequest.add(deleteRequest);
        }
        RestHighLevelClient restHighLevelClient = getRestHighLevelClient(index);
        restHighLevelClient.bulk(bulkRequest);
        return true;
    }


    @SneakyThrows
    public Object test() {
        RestClient restClient = getRestClient("");
        Response get = restClient.performRequest("get", "_cat/indices/judicial_case_v*?h=index");
        String s = EntityUtils.toString(get.getEntity());
        System.out.println(s);
        return null;
    }
}
