package com.winhc.phoenix.example.job;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.odps.Column;
import com.aliyun.odps.Odps;
import com.aliyun.odps.TableSchema;
import com.aliyun.odps.data.Record;
import com.aliyun.odps.data.RecordWriter;
import com.aliyun.odps.tunnel.TableTunnel;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import com.winhc.phoenix.example.framework.mongo.MongoDbFastScan;
import com.winhc.phoenix.example.framework.mongo.MongoDbFastScanHandle;
import com.winhc.phoenix.example.util.OdpsTableSchemaUtils;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.bson.Document;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.function.Consumer;

/**
 * @author: XuJiakai
 * 2021/12/20 09:16
 */
@Slf4j
@Component
@AllArgsConstructor
public class OdpsDataUploadJob {
    private final Odps odps;
    private final MongoTemplate mongoTemplate;

    @SneakyThrows
    public void start() {
        TableTunnel tableTunnel = new TableTunnel(odps);
        TableTunnel.UploadSession uploadSession = tableTunnel
                .createUploadSession("winhc_ng", "xjk_ads_company_hk_upload_20211220", false);
        TableSchema schema = uploadSession.getSchema();

        Consumer<List<Document>> consumer = new MongoDbFastScanHandle() {
            private RecordWriter recordWriter;
            private Record record;

            @SneakyThrows
            @Override
            public void init() {
                recordWriter = uploadSession.openBufferedWriter();
                record = uploadSession.newRecord();
            }

            @SneakyThrows
            @Override
            public void accept(List<Document> list) {
                for (Document d : list) {
                    String s = d.toJson();
                    try {
                        JSONObject jsonObject = JSON.parseObject(s);
                        JSONObject company = jsonObject.getJSONObject("company");
                        for (Column column : schema.getColumns()) {
                            String name = column.getName();
                            Object o = OdpsTableSchemaUtils.json2Value(company, name, name, schema);
                            record.set(name, o);
                        }
                    } catch (Exception e) {
                        log.info("error data: {}", s);
                        log.error(e.getMessage(), e);
                        throw new RuntimeException(e);
                    }
                    recordWriter.write(record);
                }
            }

            @SneakyThrows
            @Override
            public void close() {
                recordWriter.close();
            }
        };

        MongoDatabase db = mongoTemplate.getDb();

        MongoDbFastScan mongoDbFastScan = new MongoDbFastScan("hongkang_company_info1", consumer, db)
                .batchSize(500).threadNum(1);
        mongoDbFastScan.scan();
        uploadSession.commit();
    }

    @SneakyThrows
    public void test() {
        TableTunnel tableTunnel = new TableTunnel(odps);
        TableTunnel.UploadSession uploadSession = tableTunnel.createUploadSession("winhc_ng", "xjk_ads_company_hk_upload_20211220");
        TableSchema schema = uploadSession.getSchema();
        MongoCollection<Document> collection = mongoTemplate.getCollection("hongkang_company_info");

        for (Document d : collection.find().limit(10)) {
            String s = d.toJson();
            JSONObject jsonObject = JSON.parseObject(s);
            JSONObject company = jsonObject.getJSONObject("company");
            for (Column column : schema.getColumns()) {
                String name = column.getName();
                Object o = OdpsTableSchemaUtils.json2Value(company, name, name, schema);
                if (o == null) {
                    log.info("{} type : {} ,value : {}", name, column.getTypeInfo().getOdpsType().name(), o);
                } else {
                    log.info("{} type : {} ,value : {}", name, o.getClass().getName(), o);
                }
            }
        }

    }


}
