package com.winhc.phoenix.example.job;

import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import com.winhc.phoenix.example.configuration.DynamicElasticSearchClient;
import com.winhc.phoenix.example.framework.mongo.MongoDbFastScan;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.bson.Document;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.update.UpdateRequest;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.script.Script;
import org.elasticsearch.script.ScriptType;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.function.Consumer;

/**
 * @author: XuJiakai
 * 2021/8/13 18:30
 */
@Slf4j
@Component
@AllArgsConstructor
public class UpdateEsForMongo {

    private final DynamicElasticSearchClient dynamicElasticSearchClient;
    private final MongoTemplate mongoTemplate;

    private static final Script default_script = new Script(ScriptType.INLINE, "painless", "ctx._source.collapse_key=ctx._source.dynamic_code+'_'+ctx._source.dynamic_time", new HashMap<>());
    private static final Script deleted_script = new Script(ScriptType.INLINE, "painless", "ctx._source.deleted='5'", new HashMap<>());
    private static final Script fr_script = new Script(ScriptType.INLINE, "painless", "ctx._source.collapse_key='101003_'+ctx._source.dynamic_time", new HashMap<>());

    public void start() {
        String scanTab = "a_xjk_fix_dynamic_company_status_20210818";
        String errorTab = "a_xjk_fix_dynamic_company_status_20210818_error";

        RestHighLevelClient old = dynamicElasticSearchClient.getRestHighLevelClient("old");
        MongoDatabase db = mongoTemplate.getDb();
        MongoCollection<Document> error = db.getCollection(errorTab);
        Consumer<List<Document>> func = list -> {
            try {
                BulkRequest bulkRequest = new BulkRequest();

                list.stream().filter(r -> StringUtils.isNotEmpty(r.getString("id")))
                        .map(r -> new UpdateRequest("winhc-company-dynamic_v1", "dynamic", r.getString("id")).script(UpdateEsForMongo.deleted_script))
                        /* .map(r -> {
                             if (r.getString("dynamic_code").equals("110101")) {
                                 return new UpdateRequest("winhc-company-dynamic_v1", "dynamic", r.getString("id")).script(UpdateEsForMongo.fr_script);
                             } else {
                                 return new UpdateRequest("winhc-company-dynamic_v1", "dynamic", r.getString("id")).script(UpdateEsForMongo.default_script);
                             }
                         })*/
                        .forEach(bulkRequest::add);
                BulkResponse bulk = old.bulk(bulkRequest);
                if (bulk.hasFailures()) {
                    String s = bulk.buildFailureMessage();
                    log.error("xjk");
                    log.error(s);
                    error.insertMany(list);
                } else {
                    log.info("success ...");
                }
            } catch (Exception e) {
                error.insertMany(list);
                log.error("xjk 111");
                log.error(e.getMessage(), e);
            }
        };
        MongoDbFastScan mongoDbFastScan = new MongoDbFastScan(scanTab, func, db)
                .batchSize(500).threadNum(1);
        mongoDbFastScan.scan();
    }
}
