package com.winhc.phoenix.example.scheduled;

import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.Future;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * @author: XuJiakai
 * 2021/5/18 15:19
 */
@Slf4j
//@Component
public class TaskStatusScheduled {

    private AtomicBoolean atomicBoolean = new AtomicBoolean(false);

    private ArrayBlockingQueue<Future<Boolean>> queue = new ArrayBlockingQueue(1);


    @SneakyThrows
    public boolean isIdle() {
        while (atomicBoolean.get()) {
            log.info("put waiting ...");
            Thread.sleep(3000);
        }
        return queue.isEmpty();
    }


    @SneakyThrows
    public void put(Future<Boolean> future) {
        log.info("put future !");
        queue.put(future);
        log.info("queue size：{}", queue.size());
    }

    @Scheduled(cron = "0/20 * * * * ? ")
    public void polling() {
        log.info("task status polling ...");
        if (queue.isEmpty()) {
            return;
        }
        try {
            atomicBoolean.set(true);
            Future<Boolean> peek = queue.poll();
            if (peek.isDone()) {
                log.info("任务状态：{}", peek.get());
            } else {
                log.info("waiting ...");
                queue.put(peek);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            atomicBoolean.set(false);
        }
    }
}
