package com.winhc.phoenix.example.service.impl;

import com.winhc.phoenix.example.dao.SearchDao;
import com.winhc.phoenix.example.service.CompanyDynamicService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.InnerHitBuilder;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.collapse.CollapseBuilder;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.stereotype.Service;

import java.util.List;

import static org.elasticsearch.index.query.QueryBuilders.boolQuery;
import static org.elasticsearch.index.query.QueryBuilders.termQuery;

/**
 * @author: XuJiakai
 * 2021/7/1 14:04
 */
@Slf4j
@Service
@AllArgsConstructor
public class CompanyDynamicServiceImpl implements CompanyDynamicService {
    private final SearchDao searchDao;
    private static final String[] includes = new String[]{
            "tn"
            , "dynamic_time"
            , "rowkey"
            , "association_entity_info.*"
            , "dynamic_*"
    };

    @Override
    public Object list(String companyId, List<String> tn, int from, int size) {
        BoolQueryBuilder queryBuilder = boolQuery()
                .must(termQuery("association_entity_info.keyno", companyId));

        FetchSourceContext fetchSourceContext = new FetchSourceContext(true, includes, null);


        CollapseBuilder collapseBuilder = new CollapseBuilder("collapse_key");
        InnerHitBuilder innerHitBuilder = new InnerHitBuilder()
                .setName("collapse_record")
                .setFrom(1)
                .setSize(2)
                .setTrackScores(true)
                .setIgnoreUnmapped(true)
                .setFetchSourceContext(fetchSourceContext)
                .addSort(SortBuilders.fieldSort("rowkey").order(SortOrder.DESC));
        collapseBuilder.setInnerHits(innerHitBuilder);

        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder()
                .query(queryBuilder)
                .from(from)
                .size(size)
                .fetchSource(includes, null)
                .sort("dynamic_time", SortOrder.DESC)
                .collapse(collapseBuilder);

        return searchDao.search("winhc-company-dynamic-v1", "dynamic", searchSourceBuilder);
    }

    @Override
    public Object detail(String companyId, String collapseKey, int from, int size) {
        BoolQueryBuilder queryBuilder = boolQuery()
                .must(termQuery("association_entity_info.keyno", companyId))
                .must(termQuery("collapse_key", collapseKey));


        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder()
                .query(queryBuilder)
                .from(from)
                .size(size)
                .fetchSource(includes, null)
                .sort("rowkey", SortOrder.DESC);

        return searchDao.search("winhc-company-dynamic-v1", "dynamic", searchSourceBuilder);

    }

    @Override
    public Object debug(String code, Integer from, Integer size) {
        BoolQueryBuilder queryBuilder = boolQuery()
                .must(termQuery("dynamic_code", code));


        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder()
                .query(queryBuilder)
                .from(from)
                .size(size);

        return searchDao.search("winhc-company-dynamic_v1", "dynamic", searchSourceBuilder);

    }
}
