package com.winhc.phoenix.example.service.impl;

import cn.hutool.crypto.SecureUtil;
import com.winhc.phoenix.example.dao.SearchDao;
import com.winhc.phoenix.example.enums.CompanyQueryType;
import com.winhc.phoenix.example.enums.CompanySearchSortType;
import com.winhc.phoenix.example.service.SearchService;
import com.winhc.phoenix.example.util.company.search.CompanyQueryVo;
import com.winhc.phoenix.example.util.company.search.CompanySearchQueryUtils;
import com.winhc.phoenix.example.util.company.search.CompanySearchTipsUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;
import org.elasticsearch.search.rescore.QueryRescorerBuilder;
import org.elasticsearch.search.sort.FieldSortBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/11/19 17:08
 */
@Slf4j
@Primary
@Service(value = "v9")
@AllArgsConstructor
public class SearchV9ServiceImpl implements SearchService {

    private SearchDao searchDao;
//    public static final String index = "winhc-company-v9_2";
    public static final String index = "winhc_index_rt_company";
    public static final String type = "company";

    private static final String[] includesTips = new String[]{"cname", "legal_entity*", "estiblish_time", "reg_status_std", "company_type","logo", "new_cid"};
    private static final FetchSourceContext fetchSourceContextTips = new FetchSourceContext(true, includesTips, null);


    @Override
    public Object tips(String s) {
        QueryBuilder tips = CompanySearchTipsUtils.tips(s);
        String preference = SecureUtil.md5(s);
        List<QueryRescorerBuilder> reScoreBuilder = CompanySearchTipsUtils.getReScoreBuilder(s);
        Object search = searchDao.search(index, type, tips, reScoreBuilder, null, null, 0, 10, preference);
        return search;
    }

    @Override
    public Object controlGroup(String s) {
        return null;
    }

    @Override
    public Object query(String content, int from, int size, Set<CompanyQueryType> set, CompanySearchSortType sortType) {
        List<Integer> collect = set.stream().map(CompanyQueryType::getVal).collect(Collectors.toList());
        CompanyQueryVo companyQueryVo = CompanyQueryVo.builder()
                .content(content)
                .searchTypeList(collect)
                .build();

        //召回
        QueryBuilder queryBuilder = CompanySearchQueryUtils.getQueryBuilder(companyQueryVo);
        String preference = SecureUtil.md5(content);

        if (sortType == CompanySearchSortType.注册资本_从低到高) {
            FieldSortBuilder order = SortBuilders.fieldSort("reg_capital_amount").order(SortOrder.ASC);
            Object search = searchDao.search(index, type, queryBuilder, null, order, null, from, size, preference);
            return search;
        } else if (sortType == CompanySearchSortType.注册资本_从高到底) {
            FieldSortBuilder order = SortBuilders.fieldSort("reg_capital_amount").order(SortOrder.DESC);
            Object search = searchDao.search(index, type, queryBuilder, null, order, null, from, size, preference);
            return search;
        } else if (sortType == CompanySearchSortType.成立日期_从早到晚) {
            FieldSortBuilder order = SortBuilders.fieldSort("estiblish_time").order(SortOrder.ASC);
            Object search = searchDao.search(index, type, queryBuilder, null, order, null, from, size, preference);
            return search;
        } else if (sortType == CompanySearchSortType.成立日期_从晚到早) {
            FieldSortBuilder order = SortBuilders.fieldSort("estiblish_time").order(SortOrder.DESC);
            Object search = searchDao.search(index, type, queryBuilder, null, order, null, from, size, preference);
            return search;
        } else {
            List<QueryRescorerBuilder> reScoreBuilder = CompanySearchQueryUtils.getReScoreBuilder(companyQueryVo);
            Object search = searchDao.search(index, type, queryBuilder, reScoreBuilder, null, null, from, size, preference);
            return search;
        }
    }
}
