package com.winhc.phoenix.example.util;


import com.alibaba.fastjson.JSONObject;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.analysis.tokenattributes.CharTermAttribute;
import org.elasticsearch.index.query.*;

import java.util.Arrays;
import java.util.List;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * @author XuJiakai
 * @ClassName: QueryBuilderUtils
 * @Description:
 * @date 2019/3/25 13:03
 */
@Slf4j
public class QueryBuilderUtils {
    public static final String q = "query";

    public static WrapperQueryBuilder getQuery(JSONObject jsonObject) {
        String dsl_json;
        if (jsonObject.containsKey(q)) {
            dsl_json = JsonUtils.jsonObjToString(jsonObject.getJSONObject(q));
        } else {
            dsl_json = JsonUtils.jsonObjToString(jsonObject);
        }
        return QueryBuilders.wrapperQuery(dsl_json);
    }


    public static QueryBuilder getSpanNearQuery(String fields, String content) {
        SpanNearQueryBuilder spanNearQueryBuilder = spanNearQuery(spanTermQuery(fields, content.charAt(0) + ""), 1);
        for (int i = 1; i < content.length(); i++) {
            spanNearQueryBuilder.addClause(spanTermQuery(fields, content.charAt(i) + ""));
        }
        return spanNearQueryBuilder;
    }


    @SneakyThrows
    public static QueryBuilder spanNearQuery4StandardAnalyzer(String fields, String content) {
        try (Analyzer analyzer = new StandardAnalyzer()) {
            TokenStream stream = analyzer.tokenStream(fields, content);
            CharTermAttribute cta = stream.addAttribute(CharTermAttribute.class);
            stream.reset();

            SpanNearQueryBuilder spanNearQueryBuilder = null;
            while (stream.incrementToken()) {
                if (spanNearQueryBuilder == null) {
                    spanNearQueryBuilder = spanNearQuery(spanTermQuery(fields, cta.toString()), 0);
                } else {
                    spanNearQueryBuilder.addClause(spanTermQuery(fields, cta.toString()));
                }
            }

            if (spanNearQueryBuilder == null || spanNearQueryBuilder.clauses().size() == 1) {
                return termQuery(fields, content);
            }
            return spanNearQueryBuilder;
        }
    }


    public static QueryBuilder existField(String field) {
        return existField(field, Arrays.asList(""));
    }

    public static QueryBuilder existField(String field, List<String> ignoreStr) {
        BoolQueryBuilder must = boolQuery()
                .must(existsQuery(field));
        if (ignoreStr != null && !ignoreStr.isEmpty()) {
            BoolQueryBuilder boolQuery = boolQuery();
            for (String s : ignoreStr) {
                boolQuery.mustNot(termQuery(field, s));
            }
            must.must(boolQuery);
        }
        return must;
    }

    @SneakyThrows
    public static void main(String[] args) {
        String text = "2048";
        try (Analyzer analyzer = new StandardAnalyzer()) {
            TokenStream stream = analyzer.tokenStream("", text);
            CharTermAttribute cta = stream.addAttribute(CharTermAttribute.class);
            stream.reset();

            while (stream.incrementToken()) {
                System.out.println(cta.length());
                System.out.println(cta.toString());
            }
        }
    }
}
