package com.winhc.phoenix.example.util.company.search;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.index.query.*;
import org.elasticsearch.index.query.functionscore.ScriptScoreFunctionBuilder;
import org.elasticsearch.script.Script;
import org.elasticsearch.script.ScriptType;
import org.elasticsearch.search.rescore.QueryRescoreMode;
import org.elasticsearch.search.rescore.QueryRescorerBuilder;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * @author: XuJiakai
 * 2021/12/31 11:01
 */
@Slf4j
public class CompanySearchTipsUtils {
    public static QueryBuilder tips(String content) {

//        BoolQueryBuilder boolQuery = getBoolQuery(content);
        BoolQueryBuilder boolQuery = CompanySearchClassifyUtils.isPerson(content) ? getPersonBoolQuery(content) : getBoolQuery(content);


        //以下为过滤逻辑
        BoolQueryBuilder returnBoolQuery = boolQuery()
                .filter(termQuery("deleted", "0"))
                .filter(boolQuery().should(rangeQuery("company_score_weight").gt(0.3F))
                        .should(termsQuery("company_type", "2"))
                )
                .must(boolQuery);
        return returnBoolQuery;
    }

    private static BoolQueryBuilder getBoolQuery(String org_content) {
        log.debug("search tips keyword is not person name: {} ", org_content);
        BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();

        String content = CompanyIndexUtils.cleanup(org_content);

        if (content.length() > 3) {
            boolQuery.should(disMaxQuery()
                    .add(termQuery("history_name.show.keyword", org_content))
                    .add(termQuery("history_name.value", content))
            );

            boolQuery.should(termQuery("cname.value", content).boost(0));
            boolQuery.should(termQuery("org_number", org_content.toUpperCase()).boost(1000));
            boolQuery.should(termQuery("credit_code", org_content.toUpperCase()).boost(1000));
            boolQuery.should(termQuery("reg_number", org_content.toUpperCase()).boost(1000));
            boolQuery.should(termQuery("emails.keyword", org_content).boost(1000));
            boolQuery.should(disMaxQuery()
                    .add(termQuery("phones.keyword", org_content).boost(1000))
                    .add(matchQuery("phones", org_content).boost(1000))
            );
        }


        boolQuery.should(disMaxQuery()
                .add(disMaxQuery()
//                        .add(termQuery("legal_entity_name.keyword", org_content).boost(10))
                        .add(termQuery("legal_entities.name.keyword", org_content).boost(10))
                        .add(termQuery("holder.name.keyword", content).boost(10F))
                        .add(termQuery("holder_history.name.keyword", content).boost(10F))
                        .add(termQuery("staff.name.keyword", content).boost(5.5F))
                        .add(termQuery("staff_history.name.keyword", content).boost(5.5F))
                        .tieBreaker(0.3F))

                .tieBreaker(0.3F)
        );
        boolQuery.should(disMaxQuery()
                .add(disMaxQuery()
                        .add(termQuery("icp.keyword", org_content).boost(20))
                        .add(termQuery("app_info.keyword", org_content).boost(40))
                        .add(termQuery("company_tm.keyword", org_content).boost(20))
                        .tieBreaker(0.4F))
                .add(disMaxQuery()
                        .add(matchPhraseQuery("icp", content).boost(8).slop(3))
                        .add(matchPhraseQuery("app_info", content).boost(19).slop(3))
                        .add(matchPhraseQuery("company_tm", org_content).boost(7).slop(1))
                        .tieBreaker(0.3F))
                .tieBreaker(0.4F)
        );


        boolQuery.should(disMaxQuery()
                .add(disMaxQuery()
                        .add(matchQuery("emails", org_content).boost(7).minimumShouldMatch("100%"))
                        .add(matchQuery("icp_domain", org_content).boost(1000).minimumShouldMatch("100%"))

                        .tieBreaker(0.3F))
                .tieBreaker(0.4F)
        );

        DisMaxQueryBuilder add = disMaxQuery().add(disMaxQuery()
                .add(multiMatchQuery(content)
                        .type(MultiMatchQueryBuilder.Type.CROSS_FIELDS)
                        .minimumShouldMatch("5<90%")
                        .tieBreaker(0.3F)

                        .field("cname.show", 16)
                        .field("history_name.show", 12))
                .tieBreaker(0.4F));

        String simplifiedChinese = CompanyIndexUtils.convertToSimplifiedChinese(org_content);
        if (!StringUtils.equals(org_content, simplifiedChinese)){
            //添加繁体字简化查询
            add.add(disMaxQuery()
                    .add(multiMatchQuery(simplifiedChinese)
                            .type(MultiMatchQueryBuilder.Type.CROSS_FIELDS)
                            .minimumShouldMatch("5<90%")
                            .tieBreaker(0.3F)
                            .field("cname.simplified_chinese", 16)
                            .field("history_name.simplified_chinese", 12))
                    .tieBreaker(0.4F));
        }

        boolQuery.should(add);
        return boolQuery;
    }

    /**
     * 人员搜索
     *
     * @param org_content
     * @return
     */
    private static BoolQueryBuilder getPersonBoolQuery(String org_content) {
        log.debug("search tips keyword is person name: {} ", org_content);
        BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();

        String content = CompanyIndexUtils.cleanup(org_content);

        if (content.length() > 3) {
            boolQuery.should(disMaxQuery()
                    .add(termQuery("history_name.show.keyword", org_content))
                    .add(termQuery("history_name.value", content))
            );

            boolQuery.should(termQuery("cname.value", content).boost(0));
        }

        boolQuery.should(disMaxQuery()
                        .add(disMaxQuery()
                                        .add(termQuery("legal_entities.name.keyword", org_content).boost(10))
                                        .add(termQuery("holder.name.keyword", content).boost(10F))
                                        .add(termQuery("holder.name.keyword", org_content).boost(10F))
                                        .add(termQuery("holder_history.name.keyword", content).boost(10F))
                                        .add(termQuery("staff.name.keyword", content).boost(8F))
                                        .add(termQuery("staff_history.name.keyword", content).boost(5.5F))
                        )
                        .add(disMaxQuery()
                                        .add(matchQuery("legal_entities.name", org_content).boost(10).minimumShouldMatch("5<95%"))
                                        .add(matchPhraseQuery("holder.name", content).boost(10).slop(3))
                                        .add(matchPhraseQuery("holder_history.name", content).boost(10).slop(3))
                                        .add(matchPhraseQuery("staff.name", content).boost(8).slop(3))
                                        .add(matchPhraseQuery("staff_history.name", content).boost(6).slop(3))
                        )
        );



        DisMaxQueryBuilder add = disMaxQuery().add(disMaxQuery()
                .add(multiMatchQuery(content)
                        .operator(Operator.AND)
                        .type(MultiMatchQueryBuilder.Type.CROSS_FIELDS)
                        .tieBreaker(0.3F)
                        .field("cname.show.standard", 1)
                        .field("history_name.show.standard", 1))

                .tieBreaker(0.4F));

        String simplifiedChinese = CompanyIndexUtils.convertToSimplifiedChinese(org_content);
        if (!StringUtils.equals(org_content, simplifiedChinese)) {
            //添加繁体字简化查询
            add.add(disMaxQuery()
                    .add(disMaxQuery()
                            .add(matchPhraseQuery("cname.simplified_chinese.pinyin", simplifiedChinese))
                            .add(matchPhraseQuery("history_name.show.pinyin", simplifiedChinese))
                    )
                    .add(multiMatchQuery(simplifiedChinese)
                            .type(MultiMatchQueryBuilder.Type.CROSS_FIELDS)
                            .minimumShouldMatch("5<90%")
                            .tieBreaker(0.3F)

                            .field("cname.simplified_chinese", 1)
                            .field("history_name.simplified_chinese", 1))
                    .add(multiMatchQuery(simplifiedChinese)
                            .operator(Operator.AND)
                            .type(MultiMatchQueryBuilder.Type.CROSS_FIELDS)
                            .tieBreaker(0.3F)
                            .field("cname.simplified_chinese.standard", 1)
                            .field("history_name.simplified_chinese.standard", 1))
                    .tieBreaker(0.4F));
        }

        boolQuery.should(add);
        return boolQuery;
    }


    public static List<QueryRescorerBuilder> getReScoreBuilder(String content) {
        String c = CompanyIndexUtils.convertToSimplifiedChinese(CompanyIndexUtils.cleanup(content));
        Map<String, Object> map = new HashMap<String, Object>(2) {{
            put("query_content", c);
            put("der", 0.9);
        }};
        List<QueryRescorerBuilder> list = new ArrayList<>();

        //加上名称全匹配分数
        list.add(new QueryRescorerBuilder(functionScoreQuery(new ScriptScoreFunctionBuilder(new Script(ScriptType.STORED, null, "company-name-term-score_v2", map))))
                .windowSize(50)
                .setScoreMode(QueryRescoreMode.Total));

        //乘上权重分
        list.add(new QueryRescorerBuilder(functionScoreQuery(new ScriptScoreFunctionBuilder(new Script(ScriptType.STORED, null, "company-search-script_v3", map))))
                .windowSize(50)
                .setScoreMode(QueryRescoreMode.Multiply));
        return list;
    }

}
